import os
import subprocess
from datetime import datetime

from celery import task

from locolanderweb.models import MergeRequest, RunLog


@task
def run_project(project):
    for request in project.approved_requests():
        run_request(request)


@task
def run_request(request):
    project = request.project
    source = request.source
    target = request.target
    author = request.author
    message = request.commit_message

    cmd = [
        os.path.realpath(os.path.join(
            os.path.abspath('.'), 'docker/scripts/locolander')),
        project.name,
        source,
        target,
        message,
        author,
    ]
    try:
        output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        return_code = 0
        error = False
    except subprocess.CalledProcessError as err:
        output = err.output
        return_code = err.returncode
        error = True

    RunLog.objects.create(merge_request=request, raw=output,
                          return_code=return_code)

    # update status on merge request
    request.complete(error=error)


@task
def scan_project(project):
    mrs = project.service.approved_requests_for_project_url(project.url)
    for mr in mrs:
        # remove tzinfo and cast to UTC because USE_TZ=False
        # and sqlite3 doesn't support them
        mr['date_created'] = datetime(*mr['date_created'].timetuple()[:-2])
        MergeRequest.objects.get_or_create(
            project=project, source=mr['source'], target=mr['target'],
            date_created=mr['date_created'], defaults=mr)
